@@ -1,10 +1,10 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "duckdb-query" "0.8.0"
+(define-package "duckdb-query" "20260203.1253"
   "DuckDB query results as native Elisp data structures."
   '((emacs "28.1"))
   :url "https://github.com/gggion/duckdb-query.el"
-  :commit "b81c669859e201c4dcd1303033c0ee81134aca17"
-  :revdesc "b81c669859e2"
+  :commit "5fff3dbc9324d300ff36c9cfbd644cde36030d17"
+  :revdesc "5fff3dbc9324"
   :keywords '("data" "sql")
   :authors '(("Gino Cornejo" . "gggion123@gmail.com"))
   :maintainers '(("Gino Cornejo" . "gggion123@gmail.com")))
