@@ -5,8 +5,8 @@
 ;; Homepage: https://github.com/gggion/duckdb-query.el
 ;; Keywords: data sql
 
-;; Package-Version: 0.7.0
-;; Package-Revision: d46801ade295
+;; Package-Version: 20260203.1253
+;; Package-Revision: 5fff3dbc9324
 ;; Package-Requires: ((emacs "28.1"))
 
 ;; SPDX-License-Identifier: GPL-3.0-or-later
@@ -328,22 +328,31 @@ Also see `duckdb-query-columns' for just column names."
 
 ;;;; Context Management Macro
 
-(defmacro duckdb-query-with-database (database &rest body)
-  "Execute BODY with DATABASE as context.
+(defmacro duckdb-query-with-database (spec &rest body)
+  "Execute BODY with database from SPEC as context.
 
-DATABASE is file path string, evaluated once.
+SPEC is either:
+  - A string database path
+  - A list where first element is database path (evaluated, allows variables)
+    and remaining elements are keyword options:
+    :readonly - When non-nil, open database read-only (default nil)
 
 Behavior depends on execution context:
 
 Within session scope (inside `duckdb-query-with-session'):
-  ATTACHes DATABASE as read-only with alias derived from filename.
-  Executes BODY with database available via alias.
-  DETACHes database after BODY completes, even on error.
-  Tables accessed as ALIAS.table_name.
+  ATTACHes database with specified readonly mode.
+  Executes USE to make attached database the default catalog.
+  Executes BODY with unqualified table names resolving to database.
+  Restores previous catalog context after BODY completes.
+  DETACHes database after restoration, even on error.
+
+  Tables in the attached database can be accessed with unqualified
+  names.  Tables in the session's temp database require qualification
+  with the session catalog name.
 
 Outside session scope:
-  Binds `duckdb-query-default-database' to DATABASE for BODY.
-  All `duckdb-query' calls use DATABASE unless overridden by
+  Binds `duckdb-query-default-database' to database path for BODY.
+  All `duckdb-query' calls use database unless overridden by
   explicit :database parameter.
 
 Return value of last form in BODY.
@@ -351,33 +360,57 @@ Return value of last form in BODY.
 Example outside session:
 
   (duckdb-query-with-database \"app.db\"
-    (duckdb-query \"CREATE TABLE users (id INTEGER, name TEXT)\"
-                  :readonly nil)
-    (duckdb-query \"INSERT INTO users VALUES (1, \\='Alice\\=')\"
-                  :readonly nil)
     (duckdb-query \"SELECT * FROM users\"))
-  ;; => (((id . 1) (name . \"Alice\")))
+
+  ;; With options:
+  (duckdb-query-with-database \\='(\"app.db\" :readonly t)
+    (duckdb-query \"SELECT * FROM users\"))
+
+  ;; With variable:
+  (let ((db-path \"app.db\"))
+    (duckdb-query-with-database \\=`(,db-path :readonly t)
+      (duckdb-query \"SELECT * FROM users\")))
 
 Example within session:
 
   (duckdb-query-with-session \"work\"
     (duckdb-query-with-database \"/path/to/sales.db\"
-      ;; sales.db attached as \\='sales\\=' (filename without extension)
-      (duckdb-query \"SELECT * FROM sales.orders\")))
-  ;; Database automatically detached after body
+      ;; sales.db attached and set as default catalog
+      (duckdb-query \"SELECT * FROM orders\"))
+
+    (duckdb-query-with-database \\='(\"/path/to/logs.db\" :readonly t)
+      ;; logs.db attached read-only
+      (duckdb-query \"SELECT * FROM entries\")))
 
 Also see `duckdb-query-with-session' for session-scoped execution.
 Also see `duckdb-query-session-attach' for manual attachment."
   (declare (indent 1) (debug t))
-  (let ((db-sym (make-symbol "database"))
-        (alias-sym (make-symbol "alias")))
-    `(let ((,db-sym ,database))
+  (let ((spec-sym (make-symbol "spec"))
+        (db-sym (make-symbol "database"))
+        (readonly-sym (make-symbol "readonly"))
+        (alias-sym (make-symbol "alias"))
+        (prev-context-sym (make-symbol "prev-context")))
+    `(let* ((,spec-sym ,spec)
+            (,db-sym (if (stringp ,spec-sym)
+                         ,spec-sym
+                       (car ,spec-sym)))
+            (,readonly-sym (and (listp ,spec-sym)
+                                (plist-get (cdr ,spec-sym) :readonly))))
        (if duckdb-query--current-session
-           (let ((,alias-sym (duckdb-query-session-attach
-                              duckdb-query--current-session
-                              ,db-sym)))
+           (let* ((,alias-sym (duckdb-query-session-attach
+                               duckdb-query--current-session
+                               ,db-sym
+                               nil  ; auto-generate alias
+                               ,readonly-sym))
+                  (,prev-context-sym
+                   (format "%s.%s"
+                           (duckdb-query-value "SELECT current_catalog()")
+                           (duckdb-query-value "SELECT current_schema()"))))
+             (duckdb-query (format "USE %s.main" ,alias-sym))
              (unwind-protect
                  (progn ,@body)
+               (ignore-errors
+                 (duckdb-query (format "USE %s" ,prev-context-sym)))
                (ignore-errors
                  (duckdb-query-session-detach
                   duckdb-query--current-session
@@ -400,10 +433,11 @@ Outside session scope:
   Queries default to writable mode to enable database creation.
 
 Within session scope (inside `duckdb-query-with-session'):
-  ATTACH temp database to session with auto-generated alias.
-  Execute BODY with database available via alias.
-  DETACH and delete temp file after BODY completes.
-  Access tables as _temp_N.table_name (alias shown in return).
+  ATTACH temp database to session as writable.
+  Execute USE to make temp database the default catalog.
+  Execute BODY with unqualified names resolving to temp database.
+  Restore previous catalog context after BODY completes.
+  DETACH and delete temp file after restoration.
 
 Return value of last form in BODY.
 
@@ -421,33 +455,36 @@ Example within session:
 
   (duckdb-query-with-session \"work\"
     (duckdb-query-with-transient-database
-      ;; Temp database attached, tables accessible
       (duckdb-query \"CREATE TABLE scratch AS SELECT 1 as val\")
       (duckdb-query \"SELECT * FROM scratch\")))
   ;; Temp database automatically detached and deleted
 
-Note: For session-scoped temporary state that doesn't need file
-persistence, consider using the session's built-in temp database
-directly (tables created in session persist until session killed).
-
 Also see `duckdb-query-with-database' for named database files.
 Also see `duckdb-query-with-transient-session' for ephemeral sessions."
   (declare (indent 0) (debug t))
   (let ((db-var (make-symbol "transient-db"))
-        (alias-var (make-symbol "transient-alias")))
+        (alias-var (make-symbol "transient-alias"))
+        (prev-context-var (make-symbol "prev-context")))
     `(let ((,db-var (expand-file-name
-                     (concat (make-temp-name "duckdb-transient-")
+                     (concat (make-temp-name "duckdb_transient_")
                              ".duckdb")
                      temporary-file-directory)))
        (if duckdb-query--current-session
-           ;; Session context: ATTACH/DETACH
-           (let ((,alias-var (duckdb-query-session-attach
-                              duckdb-query--current-session
-                              ,db-var
-                              nil    ; auto-generate alias
-                              nil))) ; writable
+           ;; Session context: ATTACH/USE/DETACH
+           (let* ((,alias-var (duckdb-query-session-attach
+                               duckdb-query--current-session
+                               ,db-var
+                               nil    ; auto-generate alias
+                               nil))  ; writable for transient
+                  (,prev-context-var
+                   (format "%s.%s"
+                           (duckdb-query-value "SELECT current_catalog()")
+                           (duckdb-query-value "SELECT current_schema()"))))
+             (duckdb-query (format "USE %s.main" ,alias-var))
              (unwind-protect
                  (progn ,@body)
+               (ignore-errors
+                 (duckdb-query (format "USE %s" ,prev-context-var)))
                (ignore-errors
                  (duckdb-query-session-detach
                   duckdb-query--current-session
@@ -878,7 +915,7 @@ Signals error if session NAME already exists."
   (when (gethash name duckdb-query-sessions)
     (error "Session %s already exists" name))
   (let* ((temp-db (concat (make-temp-name
-                           (expand-file-name "duckdb-session-"
+                           (expand-file-name "duckdb_session_"
                                              temporary-file-directory))
                           ".duckdb"))
          (proc (start-process (format "duckdb-session-%s" name)
@@ -1250,8 +1287,8 @@ ALIAS is SQL identifier for the attached database.  When nil, defaults
 to filename without extension, sanitized for SQL (non-alphanumeric
 characters replaced with underscores).
 
-READONLY when non-nil (default t) opens database in read-only mode,
-preventing write conflicts.  Use nil for write access.
+READONLY when non-nil opens database in read-only mode, preventing
+write conflicts.  When nil (default), opens for read-write access.
 
 Returns alias string used for attachment.
 
@@ -1275,8 +1312,7 @@ Also see `duckdb-query-with-database' for scoped attachment."
                       (error "Session %s does not exist" session-name)))
          (proc (duckdb-session-process session))
          (raw-alias (or alias (file-name-base database)))
-         (alias (replace-regexp-in-string "[^a-zA-Z0-9_]" "_" raw-alias))
-         (readonly (if (memq readonly '(nil t)) (or readonly t) readonly)))
+         (alias (replace-regexp-in-string "[^a-zA-Z0-9_]" "_" raw-alias)))
     (unless (process-live-p proc)
       (error "Session %s process is dead" session-name))
     (duckdb-query-session-execute
